@@ -237,7 +237,7 @@ namespace acommon {
     const char * value = data_.lookup(key);
 
     if (value != 0) {
-      if (value[0] == '\x01')
+      if (value[0] == '\x01') //FIXME: Why?
 	++value;
       return String(value);
     } else {
@@ -460,7 +460,7 @@ namespace acommon {
 	    assert(second.size() == 1);
 	    unsigned int s = 0;
 	    while (s != s1.size() && s1[s] != second[0]) ++s;
-	    final_str.append(s1, s, String::npos);
+	    final_str.append(s1, s);
 	  } else if (sep == '^') {
 	    String s1 = retrieve(replace);
 	    String s2 = retrieve(second);
@@ -475,7 +475,7 @@ namespace acommon {
 
 	} else if (*i == '>') {
 
-	  final_str += retrieve(replace);
+	  final_str += retrieve(replace).data;
 	  replace = "";
 	  in_replace = false;
 
@@ -502,7 +502,6 @@ namespace acommon {
     }                                                         \
   } while (false)
 
-
   class NotifyListBlockChange : public MutableContainer 
   {
     const KeyInfo * key_info;
@@ -559,7 +558,7 @@ namespace acommon {
     RET_ON_ERR_SET(keyinfo(key), const KeyInfo *, ki);
     key = ki->name;
 
-    if (ki->otherdata[1] && attached_)
+    if (attached_ && !(ki->flags & KEYINFO_MAY_CHANGE))
       return make_err(cant_change_value, key);
   
     assert(ki->def != 0); // if null this key should never have values
@@ -682,7 +681,7 @@ namespace acommon {
     RET_ON_ERR_SET(keyinfo(k), const KeyInfo *, ki);
     const char * key = ki->name;
 
-    if (ki->otherdata[1] && attached_)
+    if (attached_ && !(ki->flags & KEYINFO_MAY_CHANGE))
       return make_err(cant_change_value, key);
   
     assert(ki->def != 0); // if null this key should never have values
@@ -858,13 +857,19 @@ namespace acommon {
     delete els;
   }
 
-  PosibErr<void> Config::read_in(IStream & in) 
+  PosibErr<void> Config::read_in(IStream & in, ParmString id) 
   {
-    FixedBuffer<> buf;
+    String buf;
     DataPair dp;
     while (getdata_pair(in, dp, buf)) {
       unescape(dp.value);
-      RET_ON_ERR(replace(dp.key, dp.value));
+      PosibErrBase pe = replace(dp.key, dp.value);
+      if (pe.has_err()) {
+        if (id.empty())
+          return pe;
+        else
+          return pe.with_file(id, dp.line_num);
+      }
     }
     return no_err;
   }
@@ -872,7 +877,7 @@ namespace acommon {
   PosibErr<void> Config::read_in_file(ParmString file) {
     FStream in;
     RET_ON_ERR(in.open(file, "r"));
-    return read_in(in);
+    return read_in(in, file);
   }
 
   PosibErr<void> Config::read_in_string(ParmString str) {
@@ -882,7 +887,6 @@ namespace acommon {
 
   void Config::merge(const Config & other) {
     StackPtr<KeyInfoEnumeration> els(possible_elements());
-    bool diff_name = strcmp(name(), other.name()) != 0;
     const KeyInfo * k;
     const KeyInfo * other_k;
     const char * other_name;
@@ -891,18 +895,8 @@ namespace acommon {
     while ( (k = els->next()) != 0) {
       if (k->type == KeyInfoDescript) continue; // FIXME: hack
 
-      if (diff_name && k->otherdata[0] == 'p'
-          && strncmp(k->name, other.name_.c_str(), other.name_.size())
-          && k->name[other.name_.size()] == '_')
-        other_name = k->name + other.name_.size();
-      else
-        other_name = k->name;
-
+      other_name = k->name;
       other_k = other.keyinfo(other_name);
-      if (diff_name && other_k && other_k->otherdata[0] == 'r') continue;
-      // the other key is a prefix key so skip it
-      // when this is a prefix key than this key
-      // would be prefix_
 
       if (other_k != 0 &&
           strcmp(k->def, other_k->def) == 0
@@ -966,8 +960,6 @@ namespace acommon {
   }
 
 
-#define CANT_CHANGE 1
-
 #ifdef ENABLE_WIN32_RELOCATABLE
 #  define HOME_DIR "<prefix>"
 #  define PERSONAL "<actual-lang>.pws"
@@ -990,35 +982,36 @@ namespace acommon {
        /* TRANSLATORS: The remaing strings in config.cpp should be kept
           under 50 characters, begin with a lower case character and not
           include any trailing punctuation marks. */
-       N_("main configuration file")             , {0, CANT_CHANGE}}
+       N_("main configuration file")}
     , {"conf-dir", KeyInfoString, CONF_DIR,
-       N_("location of main configuration file") ,{0, CANT_CHANGE}}
+       N_("location of main configuration file")}
     , {"conf-path",     KeyInfoString, "<conf-dir/conf>",     0}
     , {"data-dir", KeyInfoString, DATA_DIR,
-       N_("location of language data files"), "r"}
+       N_("location of language data files")}
     , {"dict-dir", KeyInfoString, DICT_DIR,
-       N_("location of the main word list")      }
+       N_("location of the main word list")}
     , {"encoding",   KeyInfoString, "iso8859-1",
        N_("encoding to expect data to be in")}
     , {"filter",   KeyInfoList  , "url",
-       N_("add or removes a filter")}
+       N_("add or removes a filter"), KEYINFO_MAY_CHANGE}
     , {"filter-path", KeyInfoList, FILTER_DIR,
        N_("path(es) aspell looks for filters")}
     , {"option-path", KeyInfoList, FILTER_OPT_DIR,
        N_("path(es) aspell looks for options descriptions")}
-    , {"mode",     KeyInfoString, "url",             mode_string }
+    , {"mode",     KeyInfoString, "url",
+       mode_string}
     , {"extra-dicts", KeyInfoList, "",
        N_("extra dictionaries to use")}
     , {"home-dir", KeyInfoString, HOME_DIR,
-       N_("location for personal files") }
+       N_("location for personal files")}
     , {"ignore",   KeyInfoInt   , "1",
-       N_("ignore words <= n chars")             }
+       N_("ignore words <= n chars"), KEYINFO_MAY_CHANGE}
     , {"ignore-accents" , KeyInfoBool, "false",
-       N_("ignore accents when checking words")}
+       N_("ignore accents when checking words"), KEYINFO_MAY_CHANGE}
     , {"ignore-case", KeyInfoBool  , "false",
-       N_("ignore case when checking words")}
+       N_("ignore case when checking words"), KEYINFO_MAY_CHANGE}
     , {"ignore-repl", KeyInfoBool  , "false",
-       N_("ignore commands to store replacement pairs")}
+       N_("ignore commands to store replacement pairs"), KEYINFO_MAY_CHANGE}
     , {"jargon",     KeyInfoString, "",
        N_("extra information for the word list")}
     , {"keyboard", KeyInfoString, "standard",
@@ -1038,26 +1031,26 @@ namespace acommon {
     , {"module-search-order", KeyInfoList, "",
        N_("search order for modules")}
     , {"per-conf", KeyInfoString, ".aspell.conf",
-       N_("personal configuration file"),{0, CANT_CHANGE}}
+       N_("personal configuration file")}
     , {"per-conf-path", KeyInfoString, "<home-dir/per-conf>", 0}
     , {"personal", KeyInfoString, PERSONAL,
        N_("personal dictionary file name")}
     , {"personal-path", KeyInfoString, "<home-dir/personal>", 0}
     , {"prefix",   KeyInfoString, PREFIX,
-       N_("prefix directory"), {0, CANT_CHANGE}}
+       N_("prefix directory")}
     , {"repl",     KeyInfoString, REPL,
        N_("replacements list file name") }
     , {"repl-path",     KeyInfoString, "<home-dir/repl>",     0}
     , {"run-together",        KeyInfoBool,  "false",
-       N_("consider run-together words legal")}
+       N_("consider run-together words legal"), KEYINFO_MAY_CHANGE}
     , {"run-together-limit",  KeyInfoInt,   "8",
-       N_("maxium numbers that can be strung together")}
+       N_("maxium numbers that can be strung together"), KEYINFO_MAY_CHANGE}
     , {"run-together-min",    KeyInfoInt,   "3",
-       N_("minimal length of interior words")}
+       N_("minimal length of interior words"), KEYINFO_MAY_CHANGE}
     , {"save-repl", KeyInfoBool  , "true",
-       N_("save replacement pairs on save all")}
+       N_("save replacement pairs on save all"), KEYINFO_MAY_CHANGE}
     , {"set-prefix", KeyInfoBool, "true",
-       N_("set the prefix based on executable location"), {0, CANT_CHANGE}} 
+       N_("set the prefix based on executable location")}
     , {"size",          KeyInfoString, "+60",
        N_("size of the word list")}
     , {"spelling",   KeyInfoString, "",
@@ -1065,7 +1058,7 @@ namespace acommon {
     , {"strip-accents" , KeyInfoBool, "false",
        N_("strip accents from word lists")}
     , {"sug-mode",   KeyInfoString, "normal",
-       N_("suggestion mode")}
+       N_("suggestion mode"), KEYINFO_MAY_CHANGE}
     , {"sug-edit-dist", KeyInfoInt, "1",
        N_("edit distance to use, override sug-mode default")}
     , {"sug-typo-analysis", KeyInfoBool, "true",
@@ -1073,13 +1066,15 @@ namespace acommon {
     , {"sug-repl-table", KeyInfoBool, "true",
        N_("use replacement tables, override sug-mode default")}
     , {"sug-split-chars", KeyInfoString, " -",
-       N_("characters to insert when a word is split")}
+       N_("characters to insert when a word is split"), KEYINFO_UTF8}
     , {"word-list-path", KeyInfoList, DATA_DIR,
        N_("search path for word list information files")}
     , {"affix-char",          KeyInfoString, "/", 
-       N_("indicator for affix flags in word lists")}
+       N_("indicator for affix flags in word lists"), KEYINFO_UTF8}
     , {"flag-char",           KeyInfoString, ":",
-       N_("indicator for additional flags in word lists")}
+       N_("indicator for additional flags in word lists"), KEYINFO_UTF8}
+    , {"use-other-dicts", KeyInfoBool, "true",
+       N_("use personal, replacement & session dictionaries")}
     
     //
     // These options are only used when creating dictionaries
@@ -1100,16 +1095,15 @@ namespace acommon {
        N_("create a backup file by appending \".bak\"")}
     , {"guess", KeyInfoBool, "false",
        // FIXME: shorten description
-       N_("make possible root/affix combinations not in the dictionary")} 
+       N_("make possible root/affix combinations not in the dictionary"), KEYINFO_MAY_CHANGE} 
     , {"keymapping", KeyInfoString, "aspell",
        N_("keymapping for check mode, aspell or ispell")}
     , {"reverse", KeyInfoBool, "false",
        N_("reverse the order of the suggest list")}
     , {"suggest", KeyInfoBool, "true",
-       N_("suggest possible replacements")}
+       N_("suggest possible replacements"), KEYINFO_MAY_CHANGE}
     , {"time"   , KeyInfoBool, "false",
-       N_("time load time and suggest time in pipe mode")}
-    
+       N_("time load time and suggest time in pipe mode"), KEYINFO_MAY_CHANGE}
   };
 
   const KeyInfo * config_impl_keys_begin = config_keys;
