@@ -216,7 +216,7 @@ namespace acommon {
     
     PosibErr<void> err;
 
-    FixedBuffer<> buf; DataPair d;
+    String buf; DataPair d;
     while (getdata_pair(in, d, buf)) {
       if (d.key == "order-num") {
 	to_add->c_struct.order_num = strtod_c(d.value.str, NULL);
@@ -448,7 +448,7 @@ namespace acommon {
     } else {
       FStream f;
       RET_ON_ERR(f.open(node->info_file, "r"));
-      FixedBuffer<> buf; DataPair dp; // FIXME
+      String buf; DataPair dp;
       bool res = getdata_pair(f, dp, buf);
       main_wl = dp.key; flags = dp.value;
       f.close();
