@@ -391,7 +391,7 @@ namespace acommon
     int active_option = 0;
     String expand="filter-";
     int norealoption = 0;
-    FixedBuffer<> buf; DataPair d;
+    String buf; DataPair d;
 
     if ((name_len == 6) &&
         !strncmp(key->name,"filter",6)){
@@ -588,7 +588,7 @@ namespace acommon
                   release_options(begin,begin+optsize);
                   free(begin);
                 }
-                option_value.insert(0,"(filter-)");
+                option_value.insert(0, "(filter-)", 9);
                 sprintf(line_number,"%i",line_count);
                 return make_err(identical_option,"add_filter",option_name,line_number);
               }
@@ -601,7 +601,7 @@ namespace acommon
 	      cur_opt->type = KeyInfoBool;
               cur_opt->def  = NULL;
               cur_opt->desc = NULL;
-              cur_opt->otherdata[0]='\0';
+              cur_opt->flags = 0;
               active_option = 1;
             }
             norealoption = 0;
@@ -683,15 +683,16 @@ namespace acommon
             continue;
           }
 
-	  //
-	  // key == other
-	  //
-          if (d.key == "other") {
-            strncpy(cur_opt->otherdata,option_value.c_str(),15);
-            cur_opt->otherdata[15]='\0';
+          
+ 	  //
+ 	  // key == flags
+ 	  //
+          if (d.key == "flags") {
+            if (d.value == "utf-8" || d.value == "UTF-8")
+              cur_opt->flags = KEYINFO_UTF8;
             continue;
           }
-
+           
 	  //
 	  // key == endoption
 	  //
