@@ -211,7 +211,7 @@ static inline void MAX(int & lhs, int rhs)
 PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
 {
   // io buffers
-  FixedBuffer<> buf; DataPair dp;
+  String buf; DataPair dp;
  
   // open the affix file
   affix_file = strings.dup(affpath);
@@ -233,8 +233,13 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
     /* parse in the name of the character set used by the .dict and .aff */
 
     if (dp.key == "SET") {
-      encoding = strings.dup(dp.value);
-      // FIXME: check if match
+      String buf;
+      encoding = strings.dup(fix_encoding_str(dp.value, buf));
+      char msg[96];
+      snprintf(msg, 96, _("Expected the file to be in \"%s\" not \"%s\"."),
+               lang->data_encoding(), encoding);
+      if (strcmp(encoding, lang->data_encoding()) != 0)
+        return make_err(bad_file_format, affix_file, msg);
     }
 
     /* parse in the flag used by the controlled compound words */
@@ -339,8 +344,8 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
     continue;
   error:
     char msg[32];
-    snprintf(msg, 32, _("affix '%s' is corrupt"), MsgConv(lang)(achar));
-    return make_err(bad_file_format, affix_file, msg);
+    snprintf(msg, 32, _("Affix '%s' is corrupt"), MsgConv(lang)(achar));
+    return make_err(other_error, msg).with_file(affix_file, dp.line_num);
   }
   afflst.close();
 
@@ -959,7 +964,7 @@ inline bool LookupInfo::lookup (ParmString word, WordEntry & o) const
   LookupBookkepping s;
   if (mode == Word) {
     do {
-      if (i->ws->lookup(word, o, i->cmp)) {
+      if (i->dict->lookup(word, o, i->compare)) {
         w = o.word;
         if (s.aff == 0) s.aff = o.aff;
         else append_aff(s, o); // this should not be a very common case
@@ -968,7 +973,7 @@ inline bool LookupInfo::lookup (ParmString word, WordEntry & o) const
     } while (i != end);
   } else if (mode == Soundslike) {
     do {
-      if (i->ws->soundslike_lookup(word, o)) {
+      if (i->dict->soundslike_lookup(word, o)) {
         w = o.word;
         if (s.aff == 0) s.aff = o.aff;
         else append_aff(s, o); // this should not be a very common case
