@@ -18,9 +18,9 @@
 #include <ctype.h>
 #include "settings.h"
 
-
 #ifdef USE_LOCALE
 # include <locale.h>
+# include <langinfo.h>
 #endif
 
 #include "aspell.h"
@@ -28,6 +28,13 @@
 #include <sys/types.h>
 #include <regex.h>
 
+#ifdef USE_FILE_INO
+# include <sys/types.h>
+# include <sys/stat.h>
+# include <unistd.h>
+# include <fcntl.h>
+#endif
+
 #include "asc_ctype.hpp"
 #include "check_funs.hpp"
 #include "config.hpp"
@@ -63,6 +70,7 @@ void config();
 
 void check();
 void pipe();
+void convt();
 void filter();
 void list();
 void dicts();
@@ -78,16 +86,14 @@ void dump_affix();
 
 void print_error(ParmString msg)
 {
-  fputs(_("Error: "), stderr);
-  fputs(msg, stderr);
-  fputs("\n", stderr);
+  CERR.printf(_("Error: %s\n"), msg.str());
 }
 
 void print_error(ParmString msg, ParmString str)
 {
-  fputs(_("Error: "), stderr);
-  fprintf(stderr, msg.str(), str.str());
-  fputs("\n", stderr);
+  CERR.put(_("Error: "));
+  CERR.printf(msg.str(), str.str());
+  CERR.put('\n');
 }
 
 #define EXIT_ON_ERR(command) \
@@ -153,6 +159,7 @@ const PossibleOption possible_options[] = {
   COMMAND("config",    '\0', 0),
   COMMAND("check",     'c', 0),
   COMMAND("pipe",      'a', 0),
+  COMMAND("conv",      '\0', 2),
   COMMAND("filter",    '\0', 0),
   COMMAND("soundslike",'\0', 0),
   COMMAND("munch",     '\0', 0),
@@ -231,6 +238,8 @@ int main (int argc, const char *argv[])
   const PossibleOption * o;
   const char           * parm;
 
+  Vector<StringPair> utf8_opts;
+
   //
   // process command line options by setting the appropriate options
   // in "options" and/or pushing non-options onto "argv"
@@ -245,8 +254,8 @@ int main (int argc, const char *argv[])
 	while(*c != '=' && *c != '\0') ++c;
 	o = find_option(argv[i] + 2, c);
 	if (o == possible_options_end) {
-	  option_name.assign(argv[i] + 2, 0, c - argv[i] - 2);
-	  const char * base_name = Config::base_name(option_name.c_str());
+	  option_name.assign(argv[i] + 2, c - argv[i] - 2);
+	  const char * base_name = Config::base_name(option_name);
 	  PosibErr<const KeyInfo *> ki = options->keyinfo(base_name);
           if (!ki.has_err(unknown_key)) {
             other_opt.name    = option_name.c_str();
@@ -303,7 +312,10 @@ int main (int argc, const char *argv[])
 	  args.push_back(parm);
       } else {
 	if (o->name[0] != '\0') {
-	  EXIT_ON_ERR(options->replace(o->name, parm));
+          if (options->keyinfo(Config::base_name(o->name)).data->flags & KEYINFO_UTF8)
+            utf8_opts.push_back(StringPair(strdup(o->name), strdup(parm)));
+          else
+            EXIT_ON_ERR(options->replace(o->name, parm));
 	}
       }
     } else {
@@ -319,7 +331,25 @@ int main (int argc, const char *argv[])
 
 #ifdef USE_LOCALE
   const char * codeset = nl_langinfo(CODESET);
-  if (!options->have("encoding") && codeset && !is_ascii_enc(codeset))
+  if (is_ascii_enc(codeset)) codeset = 0;
+#endif
+  if (!utf8_opts.empty()) {
+    Conv to_utf8;
+#ifdef USE_LOCALE
+    EXIT_ON_ERR(to_utf8.setup(*options, codeset, "utf-8"));
+#endif
+    for (Vector<StringPair>::iterator i = utf8_opts.begin(); 
+         i != utf8_opts.end();
+         ++i)
+    {
+      EXIT_ON_ERR(options->replace(i->first, to_utf8(i->second)));
+      free((char *)i->first);
+      free((char *)i->second);
+    }
+    utf8_opts.clear();
+  }
+#ifdef USE_LOCALE
+  if (!options->have("encoding") && codeset)
     EXIT_ON_ERR(options->replace("encoding", codeset));
 #endif
 
@@ -342,6 +372,8 @@ int main (int argc, const char *argv[])
     pipe();
   else if (action_str == "list")
     list();
+  else if (action_str == "conv")
+    convt();
   else if (action_str == "filter")
     filter();
   else if (action_str == "soundslike")
@@ -400,7 +432,7 @@ int main (int argc, const char *argv[])
 //
 
   
-static PosibErr<Convert *> setup_conv(const aspeller::Language * lang,
+static Convert * setup_conv(const aspeller::Language * lang,
                                       Config * config)
 {
   if (config->have("encoding")) {
@@ -414,8 +446,8 @@ static PosibErr<Convert *> setup_conv(const aspeller::Language * lang,
   }
 }
  
-static PosibErr<Convert *> setup_conv(Config * config,
-                                      const aspeller::Language * lang)
+static Convert * setup_conv(Config * config,
+                            const aspeller::Language * lang)
 {
   if (config->have("encoding")) {
     PosibErr<Convert *> pe = new_convert_if_needed(*config,
@@ -522,13 +554,13 @@ void print_elements(const AspellWordList * wl) {
     line += ", ";
   }
   line.resize(line.size() - 2);
-  COUT << count << ": " << line << "\n";
+  COUT.printf("%u: %s\n", count, line.c_str());
 }
 
 void status_fun(void * d, Token, int correct)
 {
   if (*static_cast<bool *>(d) && correct)
-    COUT << "*\n";
+    COUT.put("*\n");
 }
 
 DocumentChecker * new_checker(AspellSpeller * speller, 
@@ -664,11 +696,15 @@ void pipe()
 	      BREAK_ON_ERR(err = config->replace(word, word2));
             if (strcmp(word,"suggest") == 0)
               suggest = config->retrieve_bool("suggest");
+            else if (strcmp(word,"time") == 0)
+              do_time = config->retrieve_bool("time");
+            else if (strcmp(word,"guess") == 0)
+              include_guesses = config->retrieve_bool("guess");
 	    break;
 	  case 'r':
 	    word = trim_wspace(line + 4);
-	    BREAK_ON_ERR_SET(config->retrieve(word), 
-			     PosibErr<String>, ret);
+	    BREAK_ON_ERR_SET(config->retrieve(word), String, ret);
+            COUT.printl(ret);
 	    break;
 	  }
 	  break;
@@ -683,7 +719,7 @@ void pipe()
 	  }
 	  break;
 	case 'l':
-	  COUT << config->retrieve("lang") << "\n";
+	  COUT.printl(config->retrieve("lang"));
 	  break;
 	}
 	break;
@@ -705,11 +741,11 @@ void pipe()
           = aspeller::case_pattern(real_speller->lang(), cword);
         while (ci) {
           guess.clear();
-          if (ci->pre_add && ci->pre_add[0])      guess << ci->pre_add << "+";
+          if (ci->pre_add && ci->pre_add[0])      guess << ci->pre_add << '+';
           guess << ci->word;
-          if (ci->pre_strip && ci->pre_strip[0]) guess << "-" << ci->pre_strip;
-          if (ci->suf_strip && ci->suf_strip[0]) guess << "-" << ci->suf_strip;
-          if (ci->suf_add   && ci->suf_add[0])   guess << "+" << ci->suf_add;
+          if (ci->pre_strip && ci->pre_strip[0]) guess << '-' << ci->pre_strip;
+          if (ci->suf_strip && ci->suf_strip[0]) guess << '-' << ci->suf_strip;
+          if (ci->suf_add   && ci->suf_add[0])   guess << '+' << ci->suf_add;
           guesses << ", " 
                   << oconv(aspeller::fix_case(real_speller->lang(),casep, guess));
           ci = ci->next;
@@ -721,10 +757,8 @@ void pipe()
 	finish = clock();
 	if (suggestions && !aspell_word_list_empty(suggestions)) 
         {
-	  COUT << "& " << word 
-	       << " " << aspell_word_list_size(suggestions) 
-	       << " " << token.offset + ignore
-	       << ":";
+          COUT.printf("& %s %u %u:", word, aspell_word_list_size(suggestions), 
+                      token.offset + ignore);
 	  AspellStringEnumeration * els 
 	    = aspell_word_list_elements(suggestions);
 	  if (options->retrieve_bool("reverse")) {
@@ -734,37 +768,33 @@ void pipe()
 	      sugs.push_back(w);
 	    Vector<String>::reverse_iterator i = sugs.rbegin();
 	    while (true) {
-	      COUT << " " << *i;
+              COUT.printf(" %s", i->c_str());
 	      ++i;
 	      if (i == sugs.rend()) break;
-	      COUT << ",";
+              COUT.put(',');
 	    }
 	  } else {
 	    while ( ( w = aspell_string_enumeration_next(els)) != 0) {
-	      COUT << " " << w;
-	      if (!aspell_string_enumeration_at_end(els))
-		COUT << ",";
+              COUT.printf(" %s%s", w, 
+                          aspell_string_enumeration_at_end(els) ? "" : ",");
 	    }
 	  }
 	  delete_aspell_string_enumeration(els);
           if (include_guesses)
-            COUT << guesses;
-	  COUT << "\n";
+            COUT.put(guesses);
+	  COUT.put('\n');
 	} else {
           if (guesses.empty())
-            COUT << "# " << word << " " 
-                 << token.offset + ignore
-                 << "\n";
+            COUT.printf("# %s %u\n", word, token.offset + ignore);
           else
-            COUT << "? " << word << " 0 " 
-                 << token.offset + ignore
-                 << ": " << guesses.c_str() + 2;
+            COUT.printf("? %s 0 %u: %s", word, token.offset + ignore,
+                        guesses.c_str() + 2);
 	}
 	if (do_time)
-          COUT << _("Suggestion Time: ")
-               << (finish-start)/(double)CLOCKS_PER_SEC << "\n";
+          COUT.printf(_("Suggestion Time: %f\n"), 
+                      (finish-start)/(double)CLOCKS_PER_SEC);
       }
-      COUT << "\n";
+      COUT.put('\n');
     }
     if (c == EOF) break;
   }
@@ -799,6 +829,7 @@ void check()
   FILE * in = 0;
   FILE * out = 0;
   Mapping mapping;
+  bool changed = false;
 
   if (args.size() == 0) {
     print_error(_("You must specify a file name."));
@@ -815,7 +846,16 @@ void check()
     exit(-1);
   }
     
+#ifdef USE_FILE_INO
+  {
+    struct stat st;
+    fstat(fileno(in), &st);
+    int fd = open(new_name.c_str(), O_WRONLY | O_CREAT | O_TRUNC, st.st_mode);
+    if (fd >= 0) out = fdopen(fd, "w");
+  }
+#else
   out = fopen(new_name.c_str(), "w");
+#endif
   if (!out) {
     print_error(_("Could not open the file \"%s\"  for writing. File not saved."), file_name);
     exit(-1);
@@ -856,7 +896,7 @@ void check()
   menu_choices->push_back(Choice(mapping[Abort],      _("Abort")));
   menu_choices->push_back(Choice(mapping[Exit],       _("Exit")));
 
-  String new_word;
+  String word0, new_word;
   Vector<String> sug_con;
   StackPtr<StringMap> replace_list(new_string_map());
   const char * w;
@@ -865,7 +905,6 @@ void check()
 
   while (state->next_misspelling()) {
 
-    CharVector word0;
     char * word = state->get_word(word0);
 
     //
@@ -887,8 +926,7 @@ void check()
     // print the suggestions and menu choices
     //
 
-    const AspellWordList * suggestions = aspell_speller_suggest(speller, 
-                                                                word, -1);
+    const AspellWordList * suggestions = aspell_speller_suggest(speller, word, -1);
     AspellStringEnumeration * els = aspell_word_list_elements(suggestions);
     sug_con.resize(0);
     while (sug_con.size() != 10 
@@ -959,12 +997,14 @@ void check()
       if (new_word[0] >= '1' && new_word[0] < (char)suggestions_size + '1')
         new_word = sug_con[new_word[0]-'1'];
       state->replace(new_word);
+      changed = true;
       if (mapping[choice] == ReplaceAll)
         replace_list->replace(word, new_word);
       break;
     default:
       if (choice >= '1' && choice < (char)suggestions_size + '1') { 
         state->replace(sug_con[choice-'1']);
+        changed = true;
       } else {
         error(_("Sorry that is an invalid choice!"));
         goto choice_loop;
@@ -976,13 +1016,22 @@ exit_loop:
     aspell_speller_save_all_word_lists(speller);
     state.del(); // to close the file handles
     delete_aspell_speller(speller);
-    
-    bool keep_backup = options->retrieve_bool("backup");
-    String backup_name = file_name;
-    backup_name += ".bak";
-    if (keep_backup)
-      rename_file(file_name, backup_name);
-    rename_file(new_name, file_name);
+
+    if (changed) {
+
+      bool keep_backup = options->retrieve_bool("backup");
+      if (keep_backup) {
+        String backup_name = file_name;
+        backup_name += ".bak";
+        rename_file(file_name, backup_name);
+      }
+      rename_file(new_name, file_name);
+
+    } else {
+
+      remove_file(new_name);
+
+    }
     
     //end_check();
     
@@ -1090,19 +1139,34 @@ void list()
   AspellSpeller * speller = to_aspell_speller(ret);
 
   state = new CheckerString(speller,stdin,0,64);
+
+  String word;
  
   while (state->next_misspelling()) {
 
-    CharVector word0;
-    char * word = state->get_word(word0);
+    state->get_word(word);
+    COUT.printl(word);
 
-    COUT << word << "\n";
   }
   
   state.del(); // to close the file handles
   delete_aspell_speller(speller);
 }
 
+///////////////////////////
+//
+// convt
+//
+
+void convt()
+{
+  Conv conv;
+  conv.setup(*options, args[0], args[1]);
+  String line;
+  while (CIN.getline(line))
+    COUT.printl(conv(line));
+}
+
 ///////////////////////////
 //
 // filter
@@ -1123,8 +1187,8 @@ void filter()
 //
 
 void print_ver () {
-  COUT << "@(#) International Ispell Version 3.1.20 " 
-       << "(but really Aspell " << VERSION << ")" << "\n";
+  COUT.put("@(#) International Ispell Version 3.1.20 " 
+           "(but really Aspell " VERSION ")\n");
 }
 
 ///////////////////////////////////////////////////////////////////////
@@ -1157,29 +1221,28 @@ public:
   bool at_end() const {return *in;}
 };
 
-void dump (aspeller::LocalWordSet lws, Convert * conv) 
+void dump (aspeller::LocalDict lws, Convert * conv) 
 {
   using namespace aspeller;
 
-  switch (lws.word_set->basic_type) {
-  case DataSet::basic_word_set:
+  switch (lws.dict->basic_type) {
+  case Dict::basic_dict:
     {
-      BasicWordSet * ws = static_cast<BasicWordSet *>(lws.word_set);
+      Dictionary * ws = static_cast<Dictionary *>(lws.dict);
       StackPtr<WordEntryEnumeration> els(ws->detailed_elements());
       WordEntry * wi;
       while (wi = els->next(), wi) {
-        wi->write(COUT,*ws->lang(), lws.local_info.convert, conv);
+        wi->write(COUT,*ws->lang(), lws.convert, conv);
         COUT << '\n';
       }
     }
     break;
-  case DataSet::basic_multi_set:
+  case Dict::multi_dict:
     {
-      StackPtr<BasicMultiSet::Enum> els 
-	(static_cast<BasicMultiSet *>(lws.word_set)->detailed_elements());
-      LocalWordSet ws;
+      StackPtr<DictsEnumeration> els(lws.dict->dictionaries());
+      const LocalDict * ws;
       while (ws = els->next(), ws) 
-	dump (ws, conv);
+	dump (*ws, conv);
     }
     break;
   default:
@@ -1199,7 +1262,7 @@ void master () {
 
   if (action == do_create) {
     
-    EXIT_ON_ERR(create_default_readonly_word_set
+    EXIT_ON_ERR(create_default_readonly_dict
                 (new IstreamVirEnumeration(CIN),
                  *config));
 
@@ -1210,14 +1273,10 @@ void master () {
     
   } else if (action == do_dump) {
 
-    EXIT_ON_ERR_SET(add_data_set(config->retrieve("master-path"), 
-                                 *config),
-                    LoadableDataSet *, mas);
-    LocalWordSetInfo wsi;
-    wsi.set(mas->lang(), config);
-    StackPtr<Convert> conv(setup_conv(mas->lang(), config));
-    dump(LocalWordSet(mas,wsi), conv);
-    delete mas;
+    LocalDict d;
+    EXIT_ON_ERR(add_data_set(config->retrieve("master-path"), *config, d));
+    StackPtr<Convert> conv(setup_conv(d.dict->lang(), config));
+    dump(d, conv);
   }
 }
 
@@ -1230,7 +1289,7 @@ void personal () {
   using namespace aspeller;
 
   if (args.size() != 0) {
-    EXIT_ON_ERR(options->replace("personal", args[0].c_str()));
+    EXIT_ON_ERR(options->replace("personal", args[0]));
   }
   options->replace("module", "aspeller");
   if (action == do_create || action == do_merge) {
@@ -1261,17 +1320,17 @@ void personal () {
 
     StackPtr<Config> config(new_basic_config());
     EXIT_ON_ERR(config->read_in_settings(options));
-    WritableWordSet * per = new_default_writable_word_set();
-    per->load(config->retrieve("personal-path"), config);
+    Dictionary * per = new_default_writable_dict();
+    per->load(config->retrieve("personal-path"), *config);
     StackPtr<WordEntryEnumeration> els(per->detailed_elements());
-    LocalWordSetInfo wsi;
-    wsi.set(per->lang(), config);
+    LocalDictInfo wsi;
+    wsi.set(per->lang(), *config);
     StackPtr<Convert> conv(setup_conv(per->lang(), config));
 
     WordEntry * wi;
     while (wi = els->next(), wi) {
       wi->write(COUT,*(per->lang()), wsi.convert, conv);
-      COUT << "\n";
+      COUT.put('\n');
     }
     delete per;
   }
@@ -1326,8 +1385,8 @@ void repl() {
     StackPtr<Config> config(new_basic_config());
     EXIT_ON_ERR(config->read_in_settings());
 
-     WritableReplacementSet * repl = new_default_writable_replacement_set();
-     repl->load(config->retrieve("repl-path"), config);
+     ReplacementDict * repl = new_default_replacement_dict();
+     repl->load(config->retrieve("repl-path"), *config);
      StackPtr<WordEntryEnumeration> els(repl->detailed_elements());
  
      WordEntry * rl = 0;
@@ -1413,12 +1472,12 @@ void expand()
   lang.reset(res.data);
   Conv iconv(setup_conv(options, lang));
   Conv oconv(setup_conv(lang, options));
-  String word;
+  String word, buf;
   ObjStack exp_buf;
   WordAff * exp_list;
   while (CIN.getline(word)) {
-    CharVector buf; buf.append(word.c_str(), word.size() + 1);
-    char * w = iconv(buf);
+    buf = word;
+    char * w = iconv(buf.mstr(), buf.size());
     char * af = strchr(w, '/');
     size_t s;
     if (af != 0) {
@@ -1452,7 +1511,7 @@ void expand()
       for (WordAff * p = exp_list; p; p = p->next) {
         COUT << word << ' ' << oconv(p->word);
         if (p->aff[0]) COUT << '/' << oconv((const char *)p->aff);
-        if (level >= 4) COUT.print(" %f\n", ratio);
+        if (level >= 4) COUT.printf(" %f\n", ratio);
         else COUT << '\n';
       }
     }
@@ -1479,7 +1538,7 @@ static void print_wordaff(const String & base, const String & affs, Conv & oconv
   if (affs.empty())
     COUT << '\n';
   else
-    COUT.print("/%s\n", oconv(affs));
+    COUT.printf("/%s\n", oconv(affs));
 }
 
 static bool lower_equal(aspeller::Language * l, ParmString a, ParmString b)
@@ -1610,38 +1669,20 @@ void expand_expression(Config * config){
       return;
     }
     while (filterpath.expand_file_part(&seekfor,candidate)) {
-      if (((locate_ending=candidate.rfind("-filter.so")) !=
-           candidate.length() - 10)) {
-        if ((locate_ending=candidate.rfind(".flt")) !=
-            candidate.length() - 4) {
-          continue;
-        }
-        else {
-          candidate.erase(locate_ending,4);
-          eliminate_path=0;
-          while ((hold_eliminator=candidate.find('/',eliminate_path)) < 
-                 candidate.length()) {
-            eliminate_path=hold_eliminator+1;
-          }
-          toload=candidate.erase(0,eliminate_path);
-          if (regexec(&seekfor,toload.c_str(),0,NULL,0)) {
-            continue;
-          }
-        }
-      }
-      else {
-        candidate.erase(locate_ending,10);
+
+      if (candidate.suffix("-filter.so")) {
+        candidate.pop_back(10);
         eliminate_path=0;
         while ((hold_eliminator=candidate.find('/',eliminate_path)) < 
-               candidate.length()) {
+               candidate.size()) {
           eliminate_path=hold_eliminator+1;
         }
-        if (candidate.find("lib",eliminate_path) != eliminate_path) {
+        if (!candidate.prefix("lib",eliminate_path)) {
           continue;
         }
         candidate.erase(0,eliminate_path);
         candidate.erase(0,3);
-        locate_ending=candidate.length();
+        locate_ending=candidate.size();
         toload=candidate;
         if (regexec(&seekfor,toload.c_str(),0,NULL,0)) {
           continue;
@@ -1650,7 +1691,22 @@ void expand_expression(Config * config){
         if (!optionpath.expand_filename(candidate)) {
           continue;
         }
+      } else if (candidate.suffix(".flt")) {
+        candidate.pop_back(4);
+        eliminate_path=0;
+        while ((hold_eliminator=candidate.find('/',eliminate_path)) < 
+               candidate.size()) {
+          eliminate_path=hold_eliminator+1;
+        }
+        candidate.erase(0,eliminate_path);
+        toload = candidate;
+        if (regexec(&seekfor,toload.c_str(),0,NULL,0)) {
+          continue;
+        }
+      } else {
+        continue;
       }
+
       config->replace("add-filter",toload.c_str());
     }
     regfree(&seekfor);
